package com.sun.media.sound;

import java.io.SequenceInputStream;
import java.io.ByteArrayInputStream;
import java.io.DataOutputStream;
import java.io.ByteArrayOutputStream;
import javax.sound.sampled.AudioSystem;
import java.io.InputStream;
import java.io.RandomAccessFile;
import java.io.BufferedOutputStream;
import java.io.FileOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.OutputStream;
import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioFileFormat;

public final class AuFileWriter extends SunFileWriter
{
    public static final int UNKNOWN_SIZE = -1;
    
    public AuFileWriter() {
        super(new AudioFileFormat.Type[] { AudioFileFormat.Type.AU });
    }
    
    @Override
    public AudioFileFormat.Type[] getAudioFileTypes(AudioInputStream audioInputStream) {
        AudioFileFormat.Type[] array = new AudioFileFormat.Type[this.types.length];
        System.arraycopy(this.types, 0, array, 0, this.types.length);
        AudioFormat.Encoding encoding = audioInputStream.getFormat().getEncoding();
        if (AudioFormat.Encoding.ALAW.equals(encoding) || AudioFormat.Encoding.ULAW.equals(encoding) || AudioFormat.Encoding.PCM_SIGNED.equals(encoding) || AudioFormat.Encoding.PCM_UNSIGNED.equals(encoding)) {
            return array;
        }
        return new AudioFileFormat.Type[0];
    }
    
    @Override
    public int write(AudioInputStream audioInputStream, AudioFileFormat.Type type, OutputStream outputStream) throws IOException {
        return this.writeAuFile(audioInputStream, (AuFileFormat)this.getAudioFileFormat(type, audioInputStream), outputStream);
    }
    
    @Override
    public int write(AudioInputStream audioInputStream, AudioFileFormat.Type type, File file) throws IOException {
        AuFileFormat auFileFormat = (AuFileFormat)this.getAudioFileFormat(type, audioInputStream);
        BufferedOutputStream bufferedOutputStream = new BufferedOutputStream(new FileOutputStream(file), 4096);
        int writeAuFile = this.writeAuFile(audioInputStream, auFileFormat, bufferedOutputStream);
        bufferedOutputStream.close();
        if (auFileFormat.getByteLength() == -1) {
            RandomAccessFile randomAccessFile = new RandomAccessFile(file, "rw");
            if (randomAccessFile.length() <= 2147483647L) {
                randomAccessFile.skipBytes(8);
                randomAccessFile.writeInt(writeAuFile - 24);
            }
            randomAccessFile.close();
        }
        return writeAuFile;
    }
    
    private AudioFileFormat getAudioFileFormat(AudioFileFormat.Type type, AudioInputStream audioInputStream) {
        AudioFormat.Encoding pcm_SIGNED = AudioFormat.Encoding.PCM_SIGNED;
        AudioFormat format = audioInputStream.getFormat();
        AudioFormat.Encoding encoding = format.getEncoding();
        if (!this.types[0].equals(type)) {
            throw new IllegalArgumentException("File type " + type + " not supported.");
        }
        AudioFormat.Encoding encoding2;
        int sampleSizeInBits;
        if (AudioFormat.Encoding.ALAW.equals(encoding) || AudioFormat.Encoding.ULAW.equals(encoding)) {
            encoding2 = encoding;
            sampleSizeInBits = format.getSampleSizeInBits();
        }
        else if (format.getSampleSizeInBits() == 8) {
            encoding2 = AudioFormat.Encoding.PCM_SIGNED;
            sampleSizeInBits = 8;
        }
        else {
            encoding2 = AudioFormat.Encoding.PCM_SIGNED;
            sampleSizeInBits = format.getSampleSizeInBits();
        }
        AudioFormat o = new AudioFormat(encoding2, format.getSampleRate(), sampleSizeInBits, format.getChannels(), format.getFrameSize(), format.getFrameRate(), true);
        int n;
        if (audioInputStream.getFrameLength() != -1L) {
            n = (int)audioInputStream.getFrameLength() * format.getFrameSize() + 24;
        }
        else {
            n = -1;
        }
        return new AuFileFormat(AudioFileFormat.Type.AU, n, o, (int)audioInputStream.getFrameLength());
    }
    
    private InputStream getFileStream(AuFileFormat auFileFormat, InputStream audioInputStream) throws IOException {
        AudioFormat format = auFileFormat.getFormat();
        int v = 24;
        long n = (long)auFileFormat.getFrameLength();
        long n2 = (n == -1L) ? -1L : (n * (long)format.getFrameSize());
        if (n2 > 2147483647L) {
            n2 = -1L;
        }
        int auType = auFileFormat.getAuType();
        int v2 = (int)format.getSampleRate();
        int channels = format.getChannels();
        boolean bigEndian = true;
        audioInputStream = (audioInputStream = audioInputStream);
        if ((audioInputStream instanceof AudioInputStream)) {
            AudioFormat format2 = ((AudioInputStream)audioInputStream).getFormat();
            AudioFormat.Encoding encoding = format2.getEncoding();
            if (AudioFormat.Encoding.PCM_UNSIGNED.equals(encoding) || (AudioFormat.Encoding.PCM_SIGNED.equals(encoding) && bigEndian != (format2.isBigEndian() ? 1 : 0))) {
                audioInputStream = AudioSystem.getAudioInputStream(new AudioFormat(AudioFormat.Encoding.PCM_SIGNED, format2.getSampleRate(), format2.getSampleSizeInBits(), format2.getChannels(), format2.getFrameSize(), format2.getFrameRate(), (boolean)bigEndian), (AudioInputStream)audioInputStream);
            }
        }
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        DataOutputStream o = new DataOutputStream(out);
        if (!(bigEndian == 0)) {
            o.writeInt(779316836);
            o.writeInt(v);
            o.writeInt((int)n2);
            o.writeInt(auType);
            o.writeInt(v2);
            o.writeInt(channels);
        }
        else {
            o.writeInt(1684960046);
            o.writeInt(this.big2little(v));
            o.writeInt(this.big2little((int)n2));
            o.writeInt(this.big2little(auType));
            o.writeInt(this.big2little(v2));
            o.writeInt(this.big2little(channels));
        }
        o.close();
        return new SequenceInputStream(new ByteArrayInputStream(out.toByteArray()), new SunFileWriter.NoCloseInputStream(audioInputStream));
    }
    
    private int writeAuFile(InputStream inputStream, AuFileFormat auFileFormat, OutputStream outputStream) throws IOException {
        int n = 0;
        InputStream fileStream = this.getFileStream(auFileFormat, inputStream);
        byte[] array = new byte[4096];
        int byteLength = auFileFormat.getByteLength();
        int read;
        while ((read = fileStream.read(array)) >= 0) {
            if (byteLength > 0) {
                if (read < byteLength) {
                    outputStream.write(array, 0, read);
                    n += read;
                    byteLength -= read;
                }
                else {
                    outputStream.write(array, 0, byteLength);
                    n += byteLength;
                    break;
                }
            }
            else {
                outputStream.write(array, 0, read);
                n += read;
            }
        }
        return n;
    }
}
